@@ -40,14 +40,13 @@
 #include "netwm.h"
 #include "xwindow.h"
 #include "basicpholder.h"
+#include "llist.h"
+#include "bindmaps.h"
 
 
 static void set_clientwin_state(WClientWin *cwin, int state);
 static bool send_clientmsg(Window win, Atom a, Time stmp);
 
-static void do_gravity(const WRectangle *max_geom, int gravity,
-                       WRectangle *geom);
-
 
 WHook *clientwin_do_manage_alt=NULL;
 WHook *clientwin_mapped_hook=NULL;
@@ -55,7 +54,12 @@ WHook *clientwin_unmapped_hook=NULL;
 WHook *clientwin_property_change_hook=NULL;
 
 
-#define LATEST_TRANSIENT(CWIN) PTRLIST_LAST(WRegion*, (CWIN)->transient_list)
+#define LATEST_TRANSIENT(CWIN) ((CWIN)->transient_list              \
+                                ? (CWIN)->transient_list->prev->reg \
+                                : NULL)
+
+
+#define DFLT_SZPLCY SIZEPOLICY_FREE_GLUE__SOUTH
 
 
 /*{{{ Get properties */
@@ -99,47 +103,20 @@ void ioncore_set_get_winprop_fn(ExtlFn fn)
 }
 
 
-struct gravity_spec {
-    const char *spec;    /* name of the gravity value       */
-    int         gravity; /* X gravity value                 */
-};
-
-/* translation table for gravity specifications */
-static struct gravity_spec gravity_specs[] = {
-    {"center"        , CenterGravity    },
-    {"west"          , WestGravity      },
-    {"east"          , EastGravity      },
-    {"north"         , NorthGravity     },
-    {"northwest"     , NorthWestGravity },
-    {"northeast"     , NorthEastGravity },
-    {"south"         , SouthGravity     },
-    {"southwest"     , SouthWestGravity },
-    {"southeast"     , SouthEastGravity },
-    { NULL           , 0                }   /* end marker */
-};
-
-static int get_gravity_winprop(WClientWin *cwin,const char *propname)
+static WSizePolicy get_sizepolicy_winprop(WClientWin *cwin,
+                                          const char *propname,
+                                          WSizePolicy value)
 {
-    char *gravity;
-    char *s;
-    const struct gravity_spec *sp;
-    int value = ForgetGravity;
+    char *szplcy;
 
-    if(extl_table_gets_s(cwin->proptab, propname, &gravity)){
-	for(s=gravity; *s; ++s)
-	    *s = tolower(*s);
-
-	for(sp=gravity_specs; sp->spec; ++sp){
-	    if(strcmp(gravity,sp->spec)==0){
-		value=sp->gravity;
-		break;
-	    }
-	}
-	free(gravity);
+    if(extl_table_gets_s(cwin->proptab, propname, &szplcy)){
+        string2sizepolicy(szplcy, &value);
+	free(szplcy);
     }
     return value;
 }
 
+
 static void clientwin_get_winprops(WClientWin *cwin)
 {
     ExtlTab tab, tab2;
@@ -208,11 +185,11 @@ static void clientwin_get_winprops(WClientWin *cwin)
     if(extl_table_is_bool_set(tab, "ignore_cfgrq"))
         cwin->flags|=CLIENTWIN_PROP_IGNORE_CFGRQ;
 
-    if(extl_table_is_bool_set(tab, "transients_at_top"))
-        cwin->transient_gravity=NorthGravity;
-
-    cwin->gravity=get_gravity_winprop(cwin,"gravity");
-    cwin->transient_gravity=get_gravity_winprop(cwin,"transient_gravity");
+    cwin->szplcy=get_sizepolicy_winprop(cwin, "sizepolicy", 
+                                        SIZEPOLICY_DEFAULT);
+    cwin->transient_szplcy=get_sizepolicy_winprop(cwin, 
+                                                  "transient_sizepolicy",
+                                                  DFLT_SZPLCY);
 }
 
 
@@ -320,6 +297,17 @@ static void configure_cwin_bw(Window win, int bw)
 }
 
 
+static void set_sane_gravity(Window win)
+{
+    XSetWindowAttributes attr;
+    
+    attr.win_gravity=NorthWestGravity;
+    
+    XChangeWindowAttributes(ioncore_g.dpy, win,
+                            CWWinGravity, &attr);
+}
+
+
 static bool clientwin_init(WClientWin *cwin, WWindow *par, Window win,
                            XWindowAttributes *attr)
 {
@@ -346,6 +334,8 @@ static bool clientwin_init(WClientWin *cwin, WWindow *par, Window win,
         geom.y+=xgravity_deltay(cwin->size_hints.win_gravity, 
                                -cwin->orig_bw, -cwin->orig_bw);
     }
+    
+    set_sane_gravity(cwin->win);
 
     cwin->last_fp.g=geom;
     cwin->last_fp.mode=REGION_FIT_EXACT;
@@ -362,8 +352,8 @@ static bool clientwin_init(WClientWin *cwin, WWindow *par, Window win,
 
     cwin->fs_pholder=NULL;
 
-    cwin->gravity=ForgetGravity;
-    cwin->transient_gravity=ForgetGravity;
+    cwin->szplcy=SIZEPOLICY_DEFAULT;
+    cwin->transient_szplcy=DFLT_SZPLCY;
     
     region_init(&(cwin->region), par, &(cwin->last_fp));
 
@@ -661,18 +651,6 @@ void clientwin_tfor_changed(WClientWin *cwin)
 /*{{{ Add/remove managed */
 
 
-static int clientwin_get_transients_gravity(WClientWin *cwin)
-{
-    if(cwin->transient_gravity!=ForgetGravity)
-        return cwin->transient_gravity;
-    if(cwin->last_fp.mode&REGION_FIT_GRAVITY)
-        return cwin->last_fp.gravity;
-    if(cwin->gravity!=ForgetGravity)
-        return cwin->gravity;
-    return SouthGravity;
-}
-
-
 static void correct_to_size_hints_of(int *w, int *h, WRegion *reg)
 {
     XSizeHints hints;
@@ -690,74 +668,88 @@ static WRegion *clientwin_do_attach_transient(WClientWin *cwin,
 {
     WWindow *par=REGION_PARENT(cwin);
     Window bottom=None, top=None;
-    WRegion *reg, *mreg;
+    WRegion *reg=NULL, *mreg=NULL;
     WFitParams fp;
-    WRectangle mg;
+    WRectangle mg, rqgeom;
     WFrame *frame=NULL;
+    WSizePolicy szplcy;
+    WLListNode *node;
 
     if(par==NULL)
         return NULL;
 
+    node=ALLOC(WLListNode);
+    
+    if(node==NULL)
+       return NULL;
+    
     fp.mode=REGION_FIT_BOUNDS|REGION_FIT_WHATEVER;
-    fp.gravity=clientwin_get_transients_gravity(cwin);
     fp.g=cwin->last_fp.g;
 
-    if(ioncore_g.framed_transients)
+    if(ioncore_g.framed_transients){
         frame=create_frame(par, &fp, "frame-transientcontainer");
-
-    reg=fn(par, &fp, fnparams);
-    
-    if(reg==NULL){
-        destroy_obj((Obj*)frame);
-        return NULL;
+        
+        if(frame!=NULL){
+            WMPlexAttachParams param;
+            param.flags=MPLEX_ATTACH_WHATEVER;
+            
+            frame->flags|=(FRAME_DEST_EMPTY|
+                           FRAME_TAB_HIDE|
+                           FRAME_SZH_USEMINMAX|
+                           FRAME_FWD_CWIN_RQGEOM);
+            
+            reg=mplex_do_attach((WMPlex*)frame, fn, fnparams, &param);
+            
+            if(reg==NULL){
+                destroy_obj((Obj*)frame);
+            }else{
+                mreg=(WRegion*)frame;
+                mplex_managed_geom((WMPlex*)frame, &mg);
+            }
+            
+            region_remove_bindmap((WRegion*)frame, 
+                                  ioncore_mplex_toplevel_bindmap);
+            region_remove_bindmap((WRegion*)frame, 
+                                  ioncore_frame_toplevel_bindmap);
+        }
     }
 
-    if(frame!=NULL){
-        frame->flags|=(FRAME_DEST_EMPTY|
-                       FRAME_TAB_HIDE|
-                       FRAME_SZH_USEMINMAX);
-        mreg=(WRegion*)frame;
-        mplex_managed_geom((WMPlex*)frame, &mg);
-    }else{
+    if(reg==NULL){
+        reg=fn(par, &fp, fnparams);
+        if(reg==NULL){
+            free(node);
+            return NULL;
+        }
         mreg=reg;
         mg=REGION_GEOM(reg);
     }
     
-    /* border sizes */
-    fp.g.w=REGION_GEOM(mreg).w-mg.w;
-    fp.g.h=REGION_GEOM(mreg).h-mg.h;
-    /* maximum inner size */
-    mg.w=maxof(1, cwin->last_fp.g.w-fp.g.w);
-    mg.h=maxof(1, minof(REGION_GEOM(reg).h, cwin->last_fp.g.h-fp.g.h));
-    /* adjust it to size hints (can only shrink) */
-    correct_to_size_hints_of(&(mg.w), &(mg.h), reg);
-    /* final frame size */
-    fp.g.w+=mg.w;
-    fp.g.h+=mg.h;
-    /* positioning */
-    do_gravity(&(cwin->last_fp.g), fp.gravity, &(fp.g));
+    rqgeom.x=0;
+    rqgeom.y=0;
+    rqgeom.w=REGION_GEOM(reg).w;
+    rqgeom.h=REGION_GEOM(reg).h;
+    /* frame borders */
+    rqgeom.w+=REGION_GEOM(mreg).w-mg.w;
+    rqgeom.h+=REGION_GEOM(mreg).h-mg.h;
     
-    fp.mode=REGION_FIT_EXACT;
+    fp=cwin->last_fp;
+    
+    szplcy=cwin->transient_szplcy;
+    
+    sizepolicy(&szplcy, mreg, &rqgeom, REGION_RQGEOM_WEAK_ALL, &fp);
     
     region_fitrep((WRegion*)mreg, NULL, &fp);
-    
-    if(frame!=NULL){
-        if(!mplex_attach_simple((WMPlex*)frame, reg, 0)){
-            destroy_obj((Obj*)frame);
-            mreg=reg;
-        }
-    }
 
     region_stacking((WRegion*)cwin, &bottom, &top);
     region_restack(mreg, top, Above);
     
-    if(!ptrlist_insert_last(&(cwin->transient_list), mreg)){
-        if(frame!=NULL)
-            destroy_obj((Obj*)frame);
-        /* Not necessarily the right thing to do: */
-        destroy_obj((Obj*)reg);
-        return NULL;
-    }
+    node->reg=mreg;
+    node->szplcy=szplcy;
+    node->phs=NULL; /* unused */
+    node->flags=0; /* unused */
+    
+    llist_link_last(&(cwin->transient_list), node);
+    
     region_set_manager(mreg, (WRegion*)cwin);
     
     if(REGION_IS_MAPPED((WRegion*)cwin))
@@ -799,8 +791,14 @@ bool clientwin_attach_transient(WClientWin *cwin, WRegion *reg)
 static void clientwin_managed_remove(WClientWin *cwin, WRegion *transient)
 {
     bool mcf=region_may_control_focus((WRegion*)cwin);
+    WLListNode *node=llist_find_on(cwin->transient_list, transient);
+    
+    if(node==NULL)
+        return;
+    
+    llist_unlink(&(cwin->transient_list), node);
+    free(node);
     
-    ptrlist_remove(&(cwin->transient_list), transient);
     region_unset_manager(transient, (WRegion*)cwin);
     
     if(mcf){
@@ -808,20 +806,20 @@ static void clientwin_managed_remove(WClientWin *cwin, WRegion *transient)
         if(reg==NULL)
             reg=&cwin->region;
         
-        region_warp(reg);
+        region_set_focus(reg);
     }
 }
 
 
 bool clientwin_rescue_clientwins(WClientWin *cwin, WPHolder *ph)
 {
-    PtrListIterTmp tmp;
+    WLListIterTmp tmp;
     bool ret1, ret2;
     
-    ptrlist_iter_init(&tmp, cwin->transient_list);
+    llist_iter_init(&tmp, cwin->transient_list);
     
     ret1=region_rescue_some_clientwins((WRegion*)cwin, ph,
-                                       (WRegionIterator*)ptrlist_iter, 
+                                       (WRegionIterator*)llist_iter_regions, 
                                        &tmp);
 
     ret2=region_rescue_child_clientwins((WRegion*)cwin, ph);
@@ -936,14 +934,14 @@ static bool reparent_root(WClientWin *cwin)
 
 void clientwin_deinit(WClientWin *cwin)
 {
-    Obj *obj;
-    PtrListIterTmp tmp;
+    WRegion *reg;
+    WLListIterTmp tmp;
     
-    FOR_ALL_ON_PTRLIST(Obj*, obj, cwin->transient_list, tmp){
-        destroy_obj(obj);
+    FOR_ALL_REGIONS_ON_LLIST(reg, cwin->transient_list, tmp){
+        destroy_obj((Obj*)reg);
     }
     
-    assert(PTRLIST_EMPTY(cwin->transient_list));
+    assert(cwin->transient_list==NULL);
     
     if(cwin->win!=None){
         xwindow_unmanaged_selectinput(cwin->win, 0);
@@ -1184,82 +1182,11 @@ static void do_reparent_clientwin(WClientWin *cwin, Window win, int x, int y)
 }
 
 
-static void do_gravity(const WRectangle *max_geom, int gravity,
-                       WRectangle *geom)
+static void convert_transient_geom(const WFitParams *fp, WSizePolicy *szplcy,
+                                   WRegion *reg, WFitParams *resfp)
 {
-    switch(gravity){
-    case WestGravity:
-    case NorthWestGravity:
-    case SouthWestGravity:
-        geom->x=max_geom->x;
-        break;
-
-    case EastGravity:
-    case NorthEastGravity:
-    case SouthEastGravity:
-        geom->x=max_geom->x+max_geom->w-geom->w;
-        break;
-
-    default:
-        geom->x=max_geom->x+max_geom->w/2-geom->w/2;
-    }
-
-    switch(gravity){
-    case NorthGravity:
-    case NorthWestGravity:
-    case NorthEastGravity:
-        geom->y=max_geom->y;
-        break;
-
-    case SouthGravity:
-    case SouthWestGravity:
-    case SouthEastGravity:
-        geom->y=max_geom->y+max_geom->h-geom->h;
-        break;
-
-    default:
-        geom->y=max_geom->y+max_geom->h/2-geom->h/2;
-    }
-    
-    if(geom->h<=1)
-        geom->h=1;
-    if(geom->w<=1)
-        geom->w=1;
-}
-
-
-static void do_convert_geom(int htry, int gravity,
-                            const WRectangle *max_geom,
-                            WRegion *reg, WRectangle *geom)
-{
-    geom->w=max_geom->w;
-    geom->h=minof(htry, max_geom->h);
-    
-    correct_to_size_hints_of(&(geom->w), &(geom->h), reg);
-    do_gravity(max_geom, gravity, geom);
-}
-
-
-static void convert_transient_geom(const WFitParams *fp, 
-                                   WRegion *reg, WRectangle *geom)
-{
-    const WRectangle *max_geom=&(fp->g);
-    int htry=max_geom->h;
-    int gravity=ForgetGravity;
-    
-    if(fp->mode&REGION_FIT_BOUNDS){
-	if(fp->mode&REGION_FIT_GRAVITY)
-            gravity=fp->gravity;
-        if(gravity==ForgetGravity)
-            gravity=SouthGravity;
-        
-        /* TODO: old height after size hint adjustments might be
-         *       zero, while htry result in positive size.
-         */
-        htry=REGION_GEOM(reg).h;
-    }
-    
-    do_convert_geom(htry, gravity, max_geom, reg, geom);
+    *resfp=*fp;
+    sizepolicy(szplcy, reg, NULL, REGION_RQGEOM_WEAK_ALL, resfp);
 }
 
 
@@ -1267,12 +1194,14 @@ static void convert_geom(const WFitParams *fp,
                          WClientWin *cwin, WRectangle *geom)
 {
     WFitParams fptmp=*fp;
-    if(!(fptmp.mode&REGION_FIT_GRAVITY) || fptmp.gravity==ForgetGravity){
-        fptmp.mode|=REGION_FIT_GRAVITY;
-        fptmp.gravity=StaticGravity;
-    }
-
-    convert_transient_geom(&fptmp, (WRegion*)cwin, geom);
+    WSizePolicy szplcy=SIZEPOLICY_FULL_EXACT;
+    
+    if(cwin->szplcy!=SIZEPOLICY_DEFAULT)
+        szplcy=cwin->szplcy;
+    
+    sizepolicy(&szplcy, (WRegion*)cwin, NULL, REGION_RQGEOM_WEAK_ALL, &fptmp);
+    
+    *geom=fptmp.g;
 }
 
 
@@ -1286,24 +1215,21 @@ static void clientwin_managed_rqgeom(WClientWin *cwin, WRegion *sub,
                                      int flags, const WRectangle *geom, 
                                      WRectangle *geomret)
 {
-    WRectangle g, *cg=&(cwin->last_fp.g);
-
-    /* Constrain to cwin's maximum occupied area. */
-    g.w=minof(geom->w, cg->w);
-    g.h=minof(geom->h, cg->h);
+    WFitParams fp=cwin->last_fp;
+    WLListNode *node=llist_find_on(cwin->transient_list, sub);
     
-    do_gravity(cg, clientwin_get_transients_gravity(cwin), &g);
-    
-    if(!(flags&REGION_RQGEOM_WEAK_X))
-        g.x=minof(maxof(cg->x, geom->x), cg->x+cg->w-g.w);
-    if(!(flags&REGION_RQGEOM_WEAK_Y))
-        g.y=minof(maxof(cg->y, geom->y), cg->y+cg->h-g.h);
+    if(node!=NULL){
+        sizepolicy(&node->szplcy, sub, geom, flags, &fp);
+    }else{
+        WSizePolicy szplcy=SIZEPOLICY_FREE;
+        sizepolicy(&szplcy, sub, geom, flags, &fp);
+    }
     
     if(geomret!=NULL)
-        *geomret=g;
+        *geomret=fp.g;
     
     if(!(flags&REGION_RQGEOM_TRYONLY))
-        region_fit(sub, &g, REGION_FIT_EXACT);
+        region_fitrep(sub, NULL, &fp);
 }
 
 
@@ -1311,13 +1237,12 @@ static void clientwin_managed_rqgeom(WClientWin *cwin, WRegion *sub,
 static bool clientwin_fitrep(WClientWin *cwin, WWindow *np, 
                              const WFitParams *fp)
 {
-    WRegion *transient, *next;
+    WLListNode *node;
     WRectangle geom;
-    PtrListIterTmp tmp;
+    WLListIterTmp tmp;
     int diff;
     bool changes;
     int w, h;
-    WFitParams fptmp;
 
     if(np!=NULL && !region_same_rootwin((WRegion*)cwin, (WRegion*)np))
         return FALSE;
@@ -1379,21 +1304,15 @@ static bool clientwin_fitrep(WClientWin *cwin, WWindow *np,
     
     cwin->flags&=~CLIENTWIN_NEED_CFGNTFY;
     
-    fptmp.g=fp->g;
-    fptmp.mode=REGION_FIT_BOUNDS|REGION_FIT_GRAVITY;
-    if(!(fp->mode&REGION_FIT_GRAVITY) || fp->gravity==ForgetGravity)
-        fptmp.gravity=clientwin_get_transients_gravity(cwin);
-    else
-        fptmp.gravity=fp->gravity;
     
-    FOR_ALL_ON_PTRLIST(WRegion*, transient, cwin->transient_list, tmp){
+    FOR_ALL_NODES_ON_LLIST_W_NEXT(node, cwin->transient_list, tmp){
         WFitParams fp2;
-        fp2.mode=REGION_FIT_EXACT;
-        convert_transient_geom(&(fptmp), transient, &(fp2.g));
+        
+        convert_transient_geom(fp, &node->szplcy, node->reg, &fp2);
 
-        if(!region_fitrep(transient, np, &fp2) && np!=NULL){
-            warn(TR("Error reparenting %s."), region_name(transient));
-            region_detach_manager(transient);
+        if(!region_fitrep(node->reg, np, &fp2) && np!=NULL){
+            warn(TR("Error reparenting %s."), region_name(node->reg));
+            region_detach_manager(node->reg);
         }
     }
     
@@ -1404,12 +1323,12 @@ static bool clientwin_fitrep(WClientWin *cwin, WWindow *np,
 static void clientwin_map(WClientWin *cwin)
 {
     WRegion *sub;
-    PtrListIterTmp tmp;
+    WLListIterTmp tmp;
     
     show_clientwin(cwin);
     REGION_MARK_MAPPED(cwin);
     
-    FOR_ALL_ON_PTRLIST(WRegion*, sub, cwin->transient_list, tmp){
+    FOR_ALL_REGIONS_ON_LLIST(sub, cwin->transient_list, tmp){
         region_map(sub);
     }
 }
@@ -1418,12 +1337,12 @@ static void clientwin_map(WClientWin *cwin)
 static void clientwin_unmap(WClientWin *cwin)
 {
     WRegion *sub;
-    PtrListIterTmp tmp;
+    WLListIterTmp tmp;
     
     hide_clientwin(cwin);
     REGION_MARK_UNMAPPED(cwin);
     
-    FOR_ALL_ON_PTRLIST(WRegion*, sub, cwin->transient_list, tmp){
+    FOR_ALL_REGIONS_ON_LLIST(sub, cwin->transient_list, tmp){
         region_unmap(sub);
     }
 }
@@ -1456,11 +1375,11 @@ static void clientwin_do_set_focus(WClientWin *cwin, bool warp)
 static void restack_transients(WClientWin *cwin)
 {
     Window other=cwin->win;
-    PtrListIterTmp tmp;
+    WLListIterTmp tmp;
     int mode=Above;
     WRegion *reg;
 
-    FOR_ALL_ON_PTRLIST(WRegion*, reg, cwin->transient_list, tmp){
+    FOR_ALL_REGIONS_ON_LLIST(reg, cwin->transient_list, tmp){
         Window bottom=None, top=None;
         region_restack(reg, other, Above);
         region_stacking(reg, &bottom, &top);
@@ -1473,11 +1392,14 @@ static void restack_transients(WClientWin *cwin)
 
 static bool clientwin_managed_goto(WClientWin *cwin, WRegion *sub, int flags)
 {
-    if(flags&REGION_GOTO_ENTERWINDOW)
+    WLListNode *node=llist_find_on(cwin->transient_list, sub);
+    
+    if(flags&REGION_GOTO_ENTERWINDOW || node==NULL)
         return FALSE;
     
-    if(LATEST_TRANSIENT(cwin)!=sub){
-        ptrlist_reinsert_last(&(cwin->transient_list), sub);
+    if(node->next!=NULL){
+        llist_unlink(&(cwin->transient_list), node);
+        llist_link_last(&(cwin->transient_list), node);
         restack_transients(cwin);
     }
         
@@ -1504,14 +1426,14 @@ void clientwin_restack(WClientWin *cwin, Window other, int mode)
 void clientwin_stacking(WClientWin *cwin, Window *bottomret, Window *topret)
 {
     WRegion *reg;
-    PtrListIterTmp tmp;
+    WLListNode *node;
     
     *bottomret=cwin->win;
     *topret=cwin->win;
     
-    FOR_ALL_ON_PTRLIST_REV(WRegion*, reg, cwin->transient_list, tmp){
+    FOR_ALL_NODES_ON_LLIST_REV(node, cwin->transient_list){
         Window bottom=None, top=None;
-        region_stacking(reg, &bottom, &top);
+        region_stacking(node->reg, &bottom, &top);
         if(top!=None){
             *topret=top;
             break;
@@ -1535,11 +1457,11 @@ static void clientwin_activated(WClientWin *cwin)
 static void clientwin_resize_hints(WClientWin *cwin, XSizeHints *hints_ret)
 {
     WRegion *trs;
-    PtrListIterTmp tmp;
+    WLListIterTmp tmp;
     
     *hints_ret=cwin->size_hints;
     
-    FOR_ALL_ON_PTRLIST(WRegion*, trs, cwin->transient_list, tmp){
+    FOR_ALL_REGIONS_ON_LLIST(trs, cwin->transient_list, tmp){
         xsizehints_adjust_for(hints_ret, trs);
     }
 }
@@ -1721,14 +1643,15 @@ EXTL_SAFE
 EXTL_EXPORT_MEMBER
 ExtlTab clientwin_managed_list(WClientWin *cwin)
 {
-    PtrListIterTmp tmp;
-    ptrlist_iter_init(&tmp, cwin->transient_list);
+    WLListIterTmp tmp;
     
-    return extl_obj_iterable_to_table((ObjIterator*)ptrlist_iter, &tmp);
+    llist_iter_init(&tmp, cwin->transient_list);
+    
+    return extl_obj_iterable_to_table((ObjIterator*)llist_iter_regions, &tmp);
 }
 
 
-/*EXTL_DOC
+/*_EXTL_DOC
  * Toggle transients managed by \var{cwin} between top/bottom
  * of the window.
  */
@@ -1736,20 +1659,24 @@ EXTL_EXPORT_MEMBER
 void clientwin_toggle_transients_pos(WClientWin *cwin)
 {
     WRegion *transient;
-    WFitParams fp;
-    PtrListIterTmp tmp;
+    WLListNode *node;
     
-    if(cwin->transient_gravity==NorthGravity)
-	cwin->transient_gravity=SouthGravity;
-    else
-	cwin->transient_gravity=NorthGravity;
+    if((cwin->transient_szplcy&SIZEPOLICY_VERT_MASK)==SIZEPOLICY_VERT_TOP){
+        cwin->transient_szplcy&=~SIZEPOLICY_VERT_MASK;
+        cwin->transient_szplcy|=SIZEPOLICY_VERT_BOTTOM;
+    }else{
+        cwin->transient_szplcy&=~SIZEPOLICY_VERT_MASK;
+        cwin->transient_szplcy|=SIZEPOLICY_VERT_TOP;
+    }
 
-    fp.mode=REGION_FIT_BOUNDS|REGION_FIT_GRAVITY;
-    fp.g=cwin->last_fp.g;
-    fp.gravity=clientwin_get_transients_gravity(cwin);
-    
-    FOR_ALL_ON_PTRLIST(WRegion*, transient, cwin->transient_list, tmp){
-        region_fitrep(transient, NULL, &fp);
+    FOR_ALL_NODES_ON_LLIST(node, cwin->transient_list){
+        WFitParams fp2;
+        
+        node->szplcy&=~SIZEPOLICY_VERT_MASK;
+        node->szplcy|=(cwin->transient_szplcy&SIZEPOLICY_VERT_MASK);
+        
+        convert_transient_geom(&cwin->last_fp, &node->szplcy, node->reg, &fp2);
+        region_fitrep(node->reg, NULL, &fp2);
     }
 }
 
