@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/clientwin.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -22,6 +22,8 @@
 #include "attach.h"
 #include "manage.h"
 #include "pholder.h"
+#include "sizepolicy.h"
+
 
 #define CLIENTWIN_P_WM_DELETE        0x00001
 #define CLIENTWIN_P_WM_TAKE_FOCUS    0x00002
@@ -51,7 +53,7 @@ DECLCLASS(WClientWin){
 
     Window transient_for;
     
-    PtrList *transient_list;
+    WLListNode *transient_list;
     
     WClientWin *g_cwin_next, *g_cwin_prev;
     
@@ -64,13 +66,10 @@ DECLCLASS(WClientWin){
     
     WPHolder *fs_pholder;
     
-    int last_h_rq;
-    
     ExtlTab proptab;
 
-    int gravity;            /* gravity used when there is slack */
-    int transient_gravity;  /* gravity used for transients of this window  */
-                            /* if the transient's gravity is ForgetGravity */
+    WSizePolicy szplcy;           /* size policy (when there is slack) */
+    WSizePolicy transient_szplcy; /* default transient size policy */
 };
 
 
@@ -84,8 +83,15 @@ extern bool clientwin_rqclose(WClientWin *cwin, bool relocate_ignored);
 extern void clientwin_tfor_changed(WClientWin *cwin);
 
 extern bool clientwin_attach_transient(WClientWin *cwin, WRegion *transient);
-extern bool clientwin_manage_clientwin(WClientWin *cwin, WClientWin *cwin2,
-                                       const WManageParams *param, int redir);
+extern WPHolder *clientwin_prepare_manage(WClientWin *cwin, 
+                                          const WClientWin *cwin2,
+                                          const WManageParams *param, 
+                                          int redir);
+
+extern WPHolder *clientwin_prepare_manage_transient(WClientWin *cwin, 
+                                                    const WClientWin *transient,
+                                                    const WManageParams *param,
+                                                    int unused);
 
 extern void clientwin_get_set_name(WClientWin *cwin);
 
@@ -105,9 +111,9 @@ enum{
     TRANSIENT_MODE_OFF
 };
 
-extern bool clientwin_get_switchto(WClientWin *cwin);
-extern int clientwin_get_transient_mode(WClientWin *cwin);
-extern WClientWin *clientwin_get_transient_for(WClientWin *cwin);
+extern bool clientwin_get_switchto(const WClientWin *cwin);
+extern int clientwin_get_transient_mode(const WClientWin *cwin);
+extern WClientWin *clientwin_get_transient_for(const WClientWin *cwin);
 
 /* Hooks */
 
