@@ -21,6 +21,7 @@
  * warning handler.
  */
 EXTL_SAFE
+EXTL_UNTRACED
 EXTL_EXPORT
 void ioncore_warn(const char *str)
 {
@@ -28,6 +29,17 @@ void ioncore_warn(const char *str)
 }
 
 
+/*EXTL_DOC
+ * Similar to \fnref{ioncore.warn}, but also print Lua stack trace.
+ */
+EXTL_SAFE
+EXTL_EXPORT
+void ioncore_warn_traced(const char *str)
+{
+    warn("%s", str);
+}
+
+
 EXTL_SAFE
 EXTL_EXPORT
 const char *ioncore_gettext(const char *s)
