@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/modules.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2004. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -14,8 +14,15 @@
 
 #include "common.h"
 
-extern bool load_module(const char *name);
-extern void unload_modules();
-extern bool init_module_support();
+extern bool ioncore_load_module(const char *name);
+extern void ioncore_unload_modules();
+extern bool ioncore_init_module_support();
+
+typedef struct{
+    const char *name;
+    bool (*init)();
+    void (*deinit)();
+    bool loaded;
+} WStaticModuleInfo;
 
 #endif /* ION_IONCORE_MODULES_H */
