@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/mplexpholder.h
  *
- * Copyright (c) Tuomo Valkonen 2005. 
+ * Copyright (c) Tuomo Valkonen 2005-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -22,6 +22,7 @@ DECLCLASS(WMPlexPHolder){
     Watch mplex_watch;
     WLListNode *after;
     WMPlexPHolder *next, *prev;
+    int szplcy;
 };
 
 
@@ -42,8 +43,8 @@ extern void mplexpholder_deinit(WMPlexPHolder *ph);
 extern int mplexpholder_layer(WMPlexPHolder *ph);
 
 extern bool mplexpholder_do_attach(WMPlexPHolder *ph, 
-                                   WRegionAttachHandler *hnd,
-                                   void *hnd_param);
+                                   WRegionAttachHandler *hnd, void *hnd_param, 
+                                   int flags);
 
 extern bool mplexpholder_do_goto(WMPlexPHolder *ph);
 
