@@ -1,7 +1,7 @@
 /*
  * ion/libmainloop/exec.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -25,9 +25,13 @@ extern pid_t mainloop_do_spawn(const char *cmd,
                               int *infd, int *outfd, int *errfd);
 extern pid_t mainloop_spawn(const char *cmd);
 
-extern pid_t mainloop_popen_bgread(const char *cmd, ExtlFn handler,
-                                   void (*initenv)(void *p), void *p);
-extern pid_t mainloop_spawn_merr(const char *cmd, ExtlFn handler,
-                                 void (*initenv)(void *p), void *p);
+extern pid_t mainloop_popen_bgread(const char *cmd, 
+                                   void (*initenv)(void *p), void *p,
+                                   ExtlFn handler, ExtlFn errhandler);
+
+extern bool mainloop_register_input_fd_extlfn(int fd, ExtlFn fn);
+extern bool mainloop_process_pipe_extlfn(int fd, ExtlFn fn);
+
+extern void cloexec_braindamage_fix(int fd);
 
 #endif /* ION_LIBMAINLOOP_EXEC_H */
