@@ -1,7 +1,7 @@
 /*
  * ion/mod_query/query.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2006. 
  *
  * Ion is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by
@@ -49,14 +49,20 @@ WMessage *mod_query_warn(WMPlex *mplex, const char *p)
 EXTL_EXPORT
 WMessage *mod_query_message(WMPlex *mplex, const char *p)
 {
+    WMPlexAttachParams par;
+
     if(p==NULL)
         return NULL;
+    
+    par.flags=(MPLEX_ATTACH_SWITCHTO|
+               MPLEX_ATTACH_L2|
+               MPLEX_ATTACH_L2_SEMIMODAL|
+               MPLEX_ATTACH_SIZEPOLICY);
+    par.szplcy=SIZEPOLICY_FULL_BOUNDS;
 
-    return (WMessage*)mplex_attach_hnd(mplex, 
-                                       (WRegionAttachHandler*)create_wmsg,
-                                       (void*)p,
-                                       (MPLEX_ATTACH_SWITCHTO|
-                                        MPLEX_ATTACH_L2|
-                                        MPLEX_ATTACH_L2_SEMIMODAL));
+    return (WMessage*)mplex_do_attach(mplex, 
+                                      (WRegionAttachHandler*)create_wmsg,
+                                      (void*)p,
+                                      &par);
 }
 
